#ifndef __initX
#define __initX

#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/keysym.h>
#include <math.h>

#include <sys/types.h>                              
/*#include <sys/varargs.h> */
#include <unistd.h>   
#include <string.h>  
#include <stdarg.h> 
#include <pthread.h>

#define THR_STACKSIZE 10000
#define MUTEX(m) static pthread_mutex_t m = PTHREAD_MUTEX_INITIALIZER
#define LOCK(m)   pthread_mutex_lock(&(m))
#define UNLOCK(m) pthread_mutex_unlock(&(m))
#define SEM_DESTROY(s)   free(s)

/*binary semaphore:
  INIT: make s in state 1
  DOWN: atomically wait until s==1 and then set s=0
  UP:   set s=1
*/  
typedef pthread_mutex_t *Semaphore;
#define SEM_INIT(s) (s)=(pthread_mutex_t *)calloc(1,sizeof(pthread_mutex_t))
#define SEM_DOWN(s) pthread_mutex_lock(s)
#define SEM_UP(s)   pthread_mutex_unlock(s)

#define mRGB(r,g,b) (((r)<<16)|((g)<<8)|(b))
#define mLineThickness(W,t) XSetLineAttributes(theDisplay,(W)->theGC,t,LineSolid,CapProjecting,JoinMiter)
#define mSetColor(W,c) XSetForeground(theDisplay,(W)->theGC,mPixel(c))
/* set drawing mode:
 GXclear          0 
 GXand            src AND dst 
 GXandReverse     src AND NOT dst 
 GXcopy           src 
 GXandInverted    (NOT src) AND dst 
 GXnoop           dst 
 GXxor            src XOR dst 
 GXor             src OR dst 
 GXnor            (NOT src) AND (NOT dst) 
 GXequiv          (NOT src) XOR dst 
 GXinvert          NOT dst 
 GXorReverse       src OR (NOT dst) 
 GXcopyInverted    NOT src 
 GXorInverted      (NOT src) OR dst 
 GXnand            (NOT src) OR (NOT dst) 
 GXset             1 
*/
#define mSetMode(W,m) {XGCValues V; V.function=m; XChangeGC(theDisplay,(W)->theGC,GCFunction,&V);}
#define mWM_DELETE(e) ( (!strcmp(XGetAtomName(theDisplay,(e).xclient.message_type),"WM_PROTOCOLS")) &&\
                        (!strcmp(XGetAtomName(theDisplay,(Atom)(e).xclient.data.l[0]),"WM_DELETE_WINDOW")) )  
 

typedef unsigned long mColor;

typedef struct {
  /* public */
  void *storage;
  
  /* private */
  Window theWindow;   
  GC theGC;  
  int resizable;
} mWindow;

extern Display      *theDisplay;   	       /* which display on this system */
extern int           theScreen;            /* which screen number of display */
extern int	     theDepth;
extern int           theClass;             /* Class       - StaticGray 
							                                            - GrayScale 
                                          							  - StaticColor 
                                          							  - PseudoColor 
                                           							  - TrueColor 
                                          							  - DirectColor */
extern Visual 	    *theVisual; 	

/* options for opening windows */
extern  int           theXVal;             /* x position to start at  */
extern  int           theYVal;             /* y position to start at  */
extern  unsigned int  theBorderWidth;      /* width of window borders */



/* Initialize the library; 
   displayName - override the default display
   ownCmap     - boolean - use private color map 
   handlers    - initialize threads (required for handlers) ?   
*/
   
void                mInitX(char *displayName,  int ownCmap, int handlers);
void                mEndX(void);
mWindow            *mOpenWindow(int theWidth, int theHeight,int resizable,char *theTitle);
void                mCloseWindow(mWindow*);
unsigned long       mPixel(mColor color);
void                mSetBackgroundPixmap(mWindow *W,Pixmap P);
XImage             *mCreateImage(int width,int height);
void                mSetSize(mWindow *W,int width,int height);
void                mSetHandler(mWindow *W, void (*func)(mWindow *,XEvent *) );
void                mStopHandler(mWindow *W);
int                 mCreatePThread(pthread_t *newID, void *(*func)(void *), void *arg);
#endif
